
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "config.h"

#include "tmpl-enums.h"

#include "tmpl-error.h"
#include "tmpl-expr-types.h"
/* enumerations from "tmpl-error.h" */
GType
tmpl_error_get_type (void)
{
    static GType etype = 0;
    if (G_UNLIKELY(etype == 0)) {
        static const GEnumValue values[] = {
            { TMPL_ERROR_INVALID_STATE, "TMPL_ERROR_INVALID_STATE", "invalid-state" },
            { TMPL_ERROR_TEMPLATE_NOT_FOUND, "TMPL_ERROR_TEMPLATE_NOT_FOUND", "template-not-found" },
            { TMPL_ERROR_CIRCULAR_INCLUDE, "TMPL_ERROR_CIRCULAR_INCLUDE", "circular-include" },
            { TMPL_ERROR_SYNTAX_ERROR, "TMPL_ERROR_SYNTAX_ERROR", "syntax-error" },
            { TMPL_ERROR_LEXER_FAILURE, "TMPL_ERROR_LEXER_FAILURE", "lexer-failure" },
            { TMPL_ERROR_TYPE_MISMATCH, "TMPL_ERROR_TYPE_MISMATCH", "type-mismatch" },
            { TMPL_ERROR_INVALID_OP_CODE, "TMPL_ERROR_INVALID_OP_CODE", "invalid-op-code" },
            { TMPL_ERROR_DIVIDE_BY_ZERO, "TMPL_ERROR_DIVIDE_BY_ZERO", "divide-by-zero" },
            { TMPL_ERROR_MISSING_SYMBOL, "TMPL_ERROR_MISSING_SYMBOL", "missing-symbol" },
            { TMPL_ERROR_SYMBOL_REDEFINED, "TMPL_ERROR_SYMBOL_REDEFINED", "symbol-redefined" },
            { TMPL_ERROR_NOT_AN_OBJECT, "TMPL_ERROR_NOT_AN_OBJECT", "not-an-object" },
            { TMPL_ERROR_NULL_POINTER, "TMPL_ERROR_NULL_POINTER", "null-pointer" },
            { TMPL_ERROR_NO_SUCH_PROPERTY, "TMPL_ERROR_NO_SUCH_PROPERTY", "no-such-property" },
            { TMPL_ERROR_GI_FAILURE, "TMPL_ERROR_GI_FAILURE", "gi-failure" },
            { TMPL_ERROR_RUNTIME_ERROR, "TMPL_ERROR_RUNTIME_ERROR", "runtime-error" },
            { TMPL_ERROR_NOT_IMPLEMENTED, "TMPL_ERROR_NOT_IMPLEMENTED", "not-implemented" },
            { TMPL_ERROR_NOT_A_VALUE, "TMPL_ERROR_NOT_A_VALUE", "not-a-value" },
            { TMPL_ERROR_NOT_A_FUNCTION, "TMPL_ERROR_NOT_A_FUNCTION", "not-a-function" },
            { 0, NULL, NULL }
        };
        etype = g_enum_register_static (g_intern_static_string ("TmplError"), values);
    }
    return etype;
}
/* enumerations from "tmpl-expr-types.h" */
GType
tmpl_symbol_type_get_type (void)
{
    static GType etype = 0;
    if (G_UNLIKELY(etype == 0)) {
        static const GEnumValue values[] = {
            { TMPL_SYMBOL_EXPR, "TMPL_SYMBOL_EXPR", "expr" },
            { TMPL_SYMBOL_VALUE, "TMPL_SYMBOL_VALUE", "value" },
            { 0, NULL, NULL }
        };
        etype = g_enum_register_static (g_intern_static_string ("TmplSymbolType"), values);
    }
    return etype;
}
GType
tmpl_expr_type_get_type (void)
{
    static GType etype = 0;
    if (G_UNLIKELY(etype == 0)) {
        static const GEnumValue values[] = {
            { TMPL_EXPR_ADD, "TMPL_EXPR_ADD", "add" },
            { TMPL_EXPR_SUB, "TMPL_EXPR_SUB", "sub" },
            { TMPL_EXPR_MUL, "TMPL_EXPR_MUL", "mul" },
            { TMPL_EXPR_DIV, "TMPL_EXPR_DIV", "div" },
            { TMPL_EXPR_BOOLEAN, "TMPL_EXPR_BOOLEAN", "boolean" },
            { TMPL_EXPR_NUMBER, "TMPL_EXPR_NUMBER", "number" },
            { TMPL_EXPR_STRING, "TMPL_EXPR_STRING", "string" },
            { TMPL_EXPR_GT, "TMPL_EXPR_GT", "gt" },
            { TMPL_EXPR_LT, "TMPL_EXPR_LT", "lt" },
            { TMPL_EXPR_NE, "TMPL_EXPR_NE", "ne" },
            { TMPL_EXPR_EQ, "TMPL_EXPR_EQ", "eq" },
            { TMPL_EXPR_GTE, "TMPL_EXPR_GTE", "gte" },
            { TMPL_EXPR_LTE, "TMPL_EXPR_LTE", "lte" },
            { TMPL_EXPR_UNARY_MINUS, "TMPL_EXPR_UNARY_MINUS", "unary-minus" },
            { TMPL_EXPR_STMT_LIST, "TMPL_EXPR_STMT_LIST", "stmt-list" },
            { TMPL_EXPR_IF, "TMPL_EXPR_IF", "if" },
            { TMPL_EXPR_WHILE, "TMPL_EXPR_WHILE", "while" },
            { TMPL_EXPR_SYMBOL_REF, "TMPL_EXPR_SYMBOL_REF", "symbol-ref" },
            { TMPL_EXPR_SYMBOL_ASSIGN, "TMPL_EXPR_SYMBOL_ASSIGN", "symbol-assign" },
            { TMPL_EXPR_FN_CALL, "TMPL_EXPR_FN_CALL", "fn-call" },
            { TMPL_EXPR_ANON_FN_CALL, "TMPL_EXPR_ANON_FN_CALL", "anon-fn-call" },
            { TMPL_EXPR_USER_FN_CALL, "TMPL_EXPR_USER_FN_CALL", "user-fn-call" },
            { TMPL_EXPR_GETATTR, "TMPL_EXPR_GETATTR", "getattr" },
            { TMPL_EXPR_SETATTR, "TMPL_EXPR_SETATTR", "setattr" },
            { TMPL_EXPR_GI_CALL, "TMPL_EXPR_GI_CALL", "gi-call" },
            { TMPL_EXPR_REQUIRE, "TMPL_EXPR_REQUIRE", "require" },
            { TMPL_EXPR_AND, "TMPL_EXPR_AND", "and" },
            { TMPL_EXPR_OR, "TMPL_EXPR_OR", "or" },
            { TMPL_EXPR_INVERT_BOOLEAN, "TMPL_EXPR_INVERT_BOOLEAN", "invert-boolean" },
            { TMPL_EXPR_ARGS, "TMPL_EXPR_ARGS", "args" },
            { TMPL_EXPR_FUNC, "TMPL_EXPR_FUNC", "func" },
            { TMPL_EXPR_NOP, "TMPL_EXPR_NOP", "nop" },
            { TMPL_EXPR_NULL, "TMPL_EXPR_NULL", "null" },
            { 0, NULL, NULL }
        };
        etype = g_enum_register_static (g_intern_static_string ("TmplExprType"), values);
    }
    return etype;
}
GType
tmpl_expr_builtin_get_type (void)
{
    static GType etype = 0;
    if (G_UNLIKELY(etype == 0)) {
        static const GEnumValue values[] = {
            { TMPL_EXPR_BUILTIN_ABS, "TMPL_EXPR_BUILTIN_ABS", "abs" },
            { TMPL_EXPR_BUILTIN_CEIL, "TMPL_EXPR_BUILTIN_CEIL", "ceil" },
            { TMPL_EXPR_BUILTIN_FLOOR, "TMPL_EXPR_BUILTIN_FLOOR", "floor" },
            { TMPL_EXPR_BUILTIN_HEX, "TMPL_EXPR_BUILTIN_HEX", "hex" },
            { TMPL_EXPR_BUILTIN_LOG, "TMPL_EXPR_BUILTIN_LOG", "log" },
            { TMPL_EXPR_BUILTIN_PRINT, "TMPL_EXPR_BUILTIN_PRINT", "print" },
            { TMPL_EXPR_BUILTIN_REPR, "TMPL_EXPR_BUILTIN_REPR", "repr" },
            { TMPL_EXPR_BUILTIN_SQRT, "TMPL_EXPR_BUILTIN_SQRT", "sqrt" },
            { TMPL_EXPR_BUILTIN_TYPEOF, "TMPL_EXPR_BUILTIN_TYPEOF", "typeof" },
            { TMPL_EXPR_BUILTIN_ASSERT, "TMPL_EXPR_BUILTIN_ASSERT", "assert" },
            { TMPL_EXPR_BUILTIN_SIN, "TMPL_EXPR_BUILTIN_SIN", "sin" },
            { TMPL_EXPR_BUILTIN_TAN, "TMPL_EXPR_BUILTIN_TAN", "tan" },
            { TMPL_EXPR_BUILTIN_COS, "TMPL_EXPR_BUILTIN_COS", "cos" },
            { TMPL_EXPR_BUILTIN_PRINTERR, "TMPL_EXPR_BUILTIN_PRINTERR", "printerr" },
            { TMPL_EXPR_BUILTIN_CAST_BYTE, "TMPL_EXPR_BUILTIN_CAST_BYTE", "cast-byte" },
            { TMPL_EXPR_BUILTIN_CAST_CHAR, "TMPL_EXPR_BUILTIN_CAST_CHAR", "cast-char" },
            { TMPL_EXPR_BUILTIN_CAST_I32, "TMPL_EXPR_BUILTIN_CAST_I32", "cast-i32" },
            { TMPL_EXPR_BUILTIN_CAST_U32, "TMPL_EXPR_BUILTIN_CAST_U32", "cast-u32" },
            { TMPL_EXPR_BUILTIN_CAST_I64, "TMPL_EXPR_BUILTIN_CAST_I64", "cast-i64" },
            { TMPL_EXPR_BUILTIN_CAST_U64, "TMPL_EXPR_BUILTIN_CAST_U64", "cast-u64" },
            { TMPL_EXPR_BUILTIN_CAST_FLOAT, "TMPL_EXPR_BUILTIN_CAST_FLOAT", "cast-float" },
            { TMPL_EXPR_BUILTIN_CAST_DOUBLE, "TMPL_EXPR_BUILTIN_CAST_DOUBLE", "cast-double" },
            { TMPL_EXPR_BUILTIN_CAST_BOOL, "TMPL_EXPR_BUILTIN_CAST_BOOL", "cast-bool" },
            { 0, NULL, NULL }
        };
        etype = g_enum_register_static (g_intern_static_string ("TmplExprBuiltin"), values);
    }
    return etype;
}


/* Generated data ends here */

